@@ -4,7 +4,12 @@ import os
 import requests
 import sys
 import yaml
+import git
+import xxhash
 from typing import Optional, Dict
+from metricflow.cli.main import _print_issues
+from metricflow.model.parsing.config_linter import ConfigLinter
+from metricflow.model.validations.validator_helpers import ModelValidationResults
 
 EXPECTED_TF_CONFIG_FILE_NAMES = ["tdfconfig.yml", "tdfconfig.yaml", "validate_configs.yaml", "commit_configs.yaml"]
 TRANSFORM_API_URL = "https://api.transformdata.io"
@@ -29,7 +34,17 @@ def _err_msg_from_err_response(r: requests.Response) -> str:
     return err_msg
 
 
-def read_config_files(config_dir: str) -> Dict:
+def read_config_files(config_dir: str) -> Dict:  # noqa: D
+    try:
+        repo = git.Repo(".", search_parent_directories=True)
+        git_dir = repo.working_tree_dir
+    except git.exc.InvalidGitRepositoryError:
+        git_dir = None
+
+    return _read_config_files(config_dir, git_dir)
+
+
+def _read_config_files(config_dir: str, git_dir: Optional[str]) -> Dict:
     """Read yaml files from config_dir. Returns (file name, file contents) per file in dir"""
     assert os.path.exists(config_dir), f"User-specified config dir ({config_dir}) does not exist"
 
@@ -53,6 +68,8 @@ def read_config_files(config_dir: str) -> Dict:
 
             with open(os.path.join(path, fname), "r") as f:
                 filepath = os.path.abspath(os.path.join(path, fname))
+                if git_dir is not None:
+                    filepath = filepath.split(git_dir)[-1]
                 results[filepath] = f.read()
                 try:
                     yaml.safe_load_all(results[filepath])
@@ -61,11 +78,24 @@ def read_config_files(config_dir: str) -> Dict:
                 except Exception as e:
                     raise Exception(f"Failed loading yaml config file. {e}")
 
-                results[fname] = f.read()
-
     return results
 
 
+def hash_file(path: str, content: str) -> str:  # noqa: D
+    file_hash = xxhash.xxh3_128()
+    file_hash.update(path.encode())
+    file_hash.update(content.encode())
+    return file_hash.hexdigest()
+
+
+def hash_config_files(yaml_files: Dict[str, str]) -> Dict[str, str]:  # noqa: D
+    hash_to_filename = {}
+    for filename, contents in yaml_files.items():
+        hashed_file = hash_file(filename, contents)
+        hash_to_filename[hashed_file] = filename
+    return hash_to_filename
+
+
 def commit_configs(
     auth_header: Dict[str, str],
     repo: str,
@@ -74,6 +104,7 @@ def commit_configs(
     config_dir: str = LOCAL_DIR_DEFAULT,  # default to local dir
     is_validation: bool = False,
     api_url: str = TRANSFORM_API_URL,  # default to prod api
+    return_issues: bool = False,
 ) -> requests.Response:
     """Creates either a validated or validation model based on `is_validation`
 
@@ -87,24 +118,56 @@ def commit_configs(
     # false, which is why this oneliner works.
     api_url = api_url or TRANSFORM_API_URL
 
-    # get the config files
-    yaml_files = read_config_files(config_dir)
-    results = {"yaml_files": yaml_files}
-    logger.info(f"Files to upload: {yaml_files.keys()}")
-
-    # additional params for request
-    headers = {**{"Content-Type": "application/json"}, **auth_header}
-    verify = api_url.startswith("https")
-
     # Clean up branch name because people like to put slashes in their branch names
     if "/" in branch:
         branch = branch.replace("/", "__")  # dunder, for readability... to the extent it matters
 
+    # get the config files
+    yaml_files = read_config_files(config_dir)
+    hash_to_filename = hash_config_files(yaml_files)
+    yaml_hashes = list(hash_to_filename.keys())
+
+    compare_hashes_body = {"hashes": yaml_hashes}
+
+    headers = {**{"Content-Type": "application/json"}, **auth_header}
+    verify = api_url.startswith("https")
+
+    compare_hashes_url = f"{api_url}/api/v1/model/{repo}/{branch}/{commit}/compare_yaml_hashes"
+    logger.info("Uploading config file hashes")
+    r = requests.post(
+        compare_hashes_url, data=json.dumps(compare_hashes_body).encode("utf-8"), headers=headers, verify=verify
+    )
+    if r.status_code != 200:
+        err_msg = _err_msg_from_err_response(r)
+        raise Exception(err_msg)
+
+    results = r.json()
+
+    unmatched_hashes = yaml_hashes
+    if results["unmatched"] is not None:
+        unmatched_hashes = results["unmatched"]
+
+    matched_hashes = results["matched"]
+
+    upload_files = {}
+    for yaml_hash in unmatched_hashes:
+        yaml_file = hash_to_filename[yaml_hash]
+        upload_files[yaml_file] = yaml_files[yaml_file]
+
+    logger.info(f"Files to upload: {upload_files.keys()}")
+
+    add_model_files_body = {
+        "yaml_files": upload_files,
+        "yaml_hashes": matched_hashes,
+    }
+
     # Add the config files to backend file storage
     add_files_url = f"{api_url}/api/v1/model/{repo}/{branch}/{commit}/add_model_files"
     logger.info(f"add_files_url: {add_files_url}")
     logger.info("Uploading config files")
-    r = requests.post(add_files_url, data=json.dumps(results).encode("utf-8"), headers=headers, verify=verify)
+    r = requests.post(
+        add_files_url, data=json.dumps(add_model_files_body).encode("utf-8"), headers=headers, verify=verify
+    )
     if r.status_code != 200:
         raise Exception(f"Failed uploading config yaml files. {r.text}")
 
@@ -117,7 +180,7 @@ def commit_configs(
         commit_url,
         headers=headers,
         verify=verify,
-        json=json.dumps({"is_current": False, "is_validation": is_validation}),
+        json=json.dumps({"is_current": False, "is_validation": is_validation, "return_issues": return_issues}),
     )
     if r.status_code != 200:
         err_msg = _err_msg_from_err_response(r)
@@ -134,12 +197,19 @@ def commit_configs_as_primary(
     commit: str,
     config_dir: str = LOCAL_DIR_DEFAULT,  # default to local dir
     api_url: str = TRANSFORM_API_URL,  # default to prod api
+    return_issues: bool = False,
 ) -> requests.Response:
     """Creates a model from the configs and makes it the primary model
 
-    Parses configs, runs semantic validations, and creates a validated model configs
-    which is immediately made primary for the organization
+    Parses configs, runs semantic validations, and creates a validated model
+    configswhich is immediately made primary for the organization if it doesn't
+    have blocking validation issues
     """
+
+    # if return_issues == False, the http response will be an error if there
+    # are blocking issues
+    # if return_issues == True, the http response will be json and include
+    # issues
     response = commit_configs(
         auth_header=auth_header,
         repo=repo,
@@ -148,15 +218,25 @@ def commit_configs_as_primary(
         config_dir=config_dir,
         is_validation=False,
         api_url=api_url,
+        return_issues=return_issues,
     )
 
-    promote_model(
-        auth_header=auth_header,
-        repo=repo,
-        branch=branch,
-        commit=commit,
-        api_url=api_url,
-    )
+    json_resp = response.json()
+    # the only case where the resp wont have an "issues" key is if
+    # return_issues == False AND there are blocking issues during validation
+    # because in that case, the backend returns and http error response
+    if hasattr(json_resp, "issues"):
+        results = ModelValidationResults.parse_raw(json_resp["issues"])
+
+        # only promote the model if there aren't blocking issues
+        if not results.has_blocking_issues:
+            promote_model(
+                auth_header=auth_header,
+                repo=repo,
+                branch=branch,
+                commit=commit,
+                api_url=api_url,
+            )
 
     return response
 
@@ -168,6 +248,7 @@ def validate_configs(
     commit: str,
     config_dir: str = LOCAL_DIR_DEFAULT,  # default to local dir
     api_url: str = TRANSFORM_API_URL,  # default to prod api
+    return_issues: bool = False,
 ) -> requests.Response:
     """Parses configs, runs semantic validations, and creates a validation model"""
     return commit_configs(
@@ -178,6 +259,7 @@ def validate_configs(
         config_dir=config_dir,
         is_validation=True,
         api_url=api_url,
+        return_issues=return_issues,
     )
 
 
@@ -228,7 +310,7 @@ if __name__ == "__main__":
     COMMIT = os.getenv("GITHUB_SHA")
     TRANSFORM_CONFIG_DIR = os.getenv("TRANSFORM_CONFIG_DIR")
     if TRANSFORM_CONFIG_DIR:
-        TRANFSFORM_CONFIG_DIR = TRANSFORM_CONFIG_DIR.lstrip().rstrip()
+        TRANSFORM_CONFIG_DIR = TRANSFORM_CONFIG_DIR.lstrip().rstrip()
     TRANSFORM_API_KEY = os.environ["TRANSFORM_API_KEY"].lstrip().rstrip()  # fail if TRANSFORM_API_KEY not present
     auth_header = {"Authorization": f"X-Api-Key {TRANSFORM_API_KEY}"}
 
@@ -239,6 +321,11 @@ if __name__ == "__main__":
     BRANCH = f"{BRANCH}"
     COMMIT = f"{COMMIT}"
 
+    lint_results = ConfigLinter().lint_dir(TRANSFORM_CONFIG_DIR or LOCAL_DIR_DEFAULT)
+    if lint_results.has_blocking_issues:
+        _print_issues(lint_results)
+        sys.exit(1)
+
     try:
         if mode == UPLOAD_MODE_VALIDATE:
             validate_configs(
